Ext.define('App.controller.Chat', {
	extend: 'App.controller.Controller',
	
	config: {
		routes: {
			'userslist': 'listAction',
			'chat': 'chatAction' 
		},
		
		refs: {
			usersListView: {
				selector: 'usersListView',
				xtype: 'usersListView',
				autoCreate: true
			},
			chatView: {
				selector: 'chatView',
				xtype: 'chatView',
				autoCreate: true
			}
		}
	},
	
	selectedUser: 0,
	selectedUserUsername: '',
	loadMoreRequest: false,
	
	
	init: function() {
		//add handlers for socket events
		socket.on('users_list_refresh', this.list_refresh);
		socket.on('users_list_add', this.list_add);
		socket.on('users_list_remove', this.list_remove);
		socket.on('message_receive', this.message_receive);
		socket.on('get_conv_messages_resp', this.get_conv_messages);
		//get users list
		socket.emit('get_users');
	},
	
	listAction: function() {
		this.loadMoreRequest = false;
		var view = this.getUsersListView();

		var logoutBtn = view.query('#logoutButton')[0];
		var usersList = view.query('#usersList')[0];
		
		//handler for logout button
		if (!logoutBtn.hasListener('tap')) {
			logoutBtn.on('tap', function() {
				//clear local storage
				localStorage.removeItem('loggedUser');
				socket.emit('logout');
				window.location = '';
			});
		}
	
		var that = this;
		
		//if tap on a online user
		usersList.on('itemtap', function(item, index, target, record, e) {
			//save the user id and username
			that.selectedUser = record.raw.user_id;
			that.selectedUserUsername = record.raw.user_username;
			//redirect to chat
			window.location = '#chat';
		});

		this.showView(view);
	},
	
	chatAction: function() {
		//if there is no selected user
		if (0 >= this.selectedUser) {
			//redirect to users list
			window.location = '#userslist';
			return;
		}
		
		//get instance of messages store
		var store = Ext.getStore('Messages');
		
		//filter the messages with the selected user id
		store.filterByUser(this.selectedUser);
		
		//if no messages loaded
		if (0 == store.getCount()) {
			//load last 10 messages
			this.loadMoreRequest++;
			socket.emit('get_conv_messages', this.selectedUser, 0, 10);
		}
		
		var view = this.getChatView();
		
		var messageBtn = view.query('#messageButton')[0];
		var messageInp = view.query('#messageInput')[0];
		var topBar = view.query('#topChatBar')[0];
		var backBtn = view.query('#backButton')[0];
		var listItem = view.query('#messageList')[0];
		topBar.setTitle(this.selectedUserUsername);
		var that = this;
		
		//handler for back button
		if (!backBtn.hasListener('tap')) {
			backBtn.on('tap', function() {
				that.selectedUser = 0;
				that.selectedUserUsername = '';
				window.location = '#userslist';
			});
		}
		
		//handler for send message button
		if (!messageBtn.hasListener('tap')) {
			messageBtn.on('tap', function() {
				var message_content = messageInp.getValue().trim();
				message_content = message_content.split("\n").join("<br/>");
				//if message content empty, stop function
				if (!message_content) {
					return;
				}
				
				//get messages store instance
				var store = Ext.StoreManager.get('Messages');
				
				//create new message object
				var newMessage = new App.model.Message(
					{
						message_content: message_content,
						sent: true,
						message_with: that.selectedUser,
						message_time: (new Date().getTime()/1000),
						message_read: 1
					});
					
				//add new message to messages store
				store.add(newMessage);
				
				//scroll to last message after a short delay
				setTimeout(function() {
					that.getChatView().query('#messageList')[0].scrollToRecord(newMessage);
				}, 50);
				
				
				socket.emit('message_sent', that.selectedUser, message_content);
				messageInp.setValue('');
			});
		}
		
		//handler for list scroll
		listItem.getScrollable()._scroller.on('scroll', function() {
			//if the scroll reaches a point and there was not another request made
			if (-30 >= arguments[2] && !that.loadMoreRequest) {
				//request more messages
				that.loadMoreRequest = true;
				socket.emit('get_conv_messages', that.selectedUser, store.getCount(), 10);
			}
		});
		
		//read all messages for this user
		this.readMessages();
		
		this.showView(view);
	},
	
	//helpers
	readMessages: function() {
		//get instances of messages and online users stores
		var messagesStore = Ext.StoreManager.get('Messages');
		var usersStore = Ext.StoreManager.get('OnlineUsers');
		
		//read all messages from user
		messagesStore.readMessages(this.selectedUser);
		usersStore.readMessages(this.selectedUser);
	},
	
	//handlers
	//refresh list of users
	list_refresh: function(list) {
		//parse the list of users
		list = JSON.parse(list);
		delete list[socket.socket.sessionid];
		
		//get instance of online users store
		var store = Ext.StoreManager.get('OnlineUsers');
		//remove all elements of store
		store.removeAll();
		
		//add the new users to store
		for (var key in list) {
			if (list[key].user_username) {
				store.add(new App.model.OnlineUser(
					{
						user_username: list[key].user_username,
						user_id: list[key].user_id
					}
				));
			}
		}
	},
	
	//add online user
	list_add: function(user_username, user_id) {
		var store = Ext.StoreManager.get('OnlineUsers');
		
		//add new online user object to store
		store.add(new App.model.OnlineUser(
			{
				user_username: user_username,
				user_id: user_id
			}
		));
	},
	
	//remove online user
	list_remove: function(user_id) {
		//remove online user by id
		Ext.StoreManager.get('OnlineUsers').removeUser(user_id);
	},
	
	//handle new message event
	message_receive: function(message_from, message_content, message_id) {
		//get instances of messages and online users stores
		var store = Ext.StoreManager.get('Messages');
		var usersStore = Ext.StoreManager.get('OnlineUsers');	
				
		message_read = 0;

		//if current screen is the chat one and the selected user is the sender of the message
		if ('#chat' == window.location.hash) {
			if (message_from == App.app.getController('Chat').selectedUser) {
				//mark this message as read
				message_read = 1;
			}
		}
				
		//create new message object
		var newMessage = new App.model.Message(
			{
				message_content: message_content,
				sent: false,
				message_with: message_from,
				message_time: (new Date().getTime()/1000),
				message_read: 0,
				message_id: message_id
			}
		);
				
		//add new object to store
		store.add(newMessage);
				
		//if message should not be marked as read
		if (0 == message_read) {
			//add to unread messages
			usersStore.addUnreadMessage(message_from);
		}
		else {
			//set message as read
			store.readMessages(App.app.getController('Chat').selectedUser);
			//scroll to last message
			setTimeout(function() {
				App.app.getController('Chat').getChatView().query('#messageList')[0].scrollToRecord(newMessage);
			}, 50);	
		}	
	},
	
	//get the conversation messages
	get_conv_messages: function(messages_json) {
		//decode messages json
		var messages_array = JSON.parse(messages_json);
		
		var store = Ext.StoreManager.get('Messages');
		var selectedUser = App.app.getController('Chat').selectedUser;
		
		//parse messages
		for (var i in messages_array) {
			var message_content = messages_array[i]['message_content'];
			var message_time = messages_array[i]['message_time'];
			var sent = false;
			var message_read = messages_array[i]['message_read'];
			if (selectedUser == messages_array[i]['message_to']) {
				sent = true;
				message_read = 1;
			}
			var message_id = messages_array[i]['message_id'];
			
			//insert in store new message object
			store.insert(0,[new App.model.Message(
				{
					message_content: message_content,
					sent: sent,
					message_with: selectedUser,
					message_time: message_time,
					message_read: message_read,
					message_id: message_id
				}
			)]);
		}//for (var i in messages_array)
		
		//accept load more messages after 1 sec
		setTimeout(function() {
			App.app.getController('Chat').loadMoreRequest = false;	
		}, 1000);
		
	}
});
