Ext.define('App.controller.Main', {
	extend: 'App.controller.Controller',
	
	config: {
		routes: {
			'': 'homeAction',
			'home': 'homeAction',
			'login': 'loginAction',
			'register': 'registerAction'
		},
		
		refs: {
			loginView: {
				selector: 'loginView',
				xtype: 'loginView',
				autoCreate: true
			},
			registerView: {
				selector: 'registerView',
				xtype: 'registerView',
				autoCreate: true
			}
		}
	},
	
	init: function() {
		//attach handlers for socket events
		socket.on('register_user_response', this.register_response);
		socket.on('login_user_response', this.login_response);
		socket.on('auto_login_response', this.auto_login_response);
	},
	
	homeAction: function() {
		//get logged user from local storage
		var loggedUser = localStorage.getItem('loggedUser');
		
		//if there is no logged user
		if (null === loggedUser) {
			//redirect to login page
			window.location = '#login';
			//stop the function
			return;
		}
		
		//if user found, login as that user
		var object = JSON.parse(loggedUser);
		
		socket.emit('auto_login', object.user_id);
	},
	
	loginAction: function() {
		var view = this.getLoginView();
		
		var registerBtn = view.query('#registerButton')[0];
		var usernameInp = view.query('#usernameField')[0];
		var passwordInp = view.query('#passwordField')[0];
		var loginBtn = view.query('#loginButton')[0];
		
		//handler for login button
		if (!loginBtn.hasListener('tap')) {
			loginBtn.on('tap', function() {
				var username = usernameInp.getValue().trim(),
					password = passwordInp.getValue();
					
				if (!username) {
					alert('You must insert a username');
					return;
				}
				
				if (!password) {
					alert('You must insert a password');
					return;
				}
				
				socket.emit('login', username, password);
			});
		}
		
		//handler for register button
		if (!registerBtn.hasListener('tap')) {
			registerBtn.on('tap', function() {
				window.location = "#register";
			});
		}
		
		this.showView(view);
	},
	
	registerAction: function() {
		var view = this.getRegisterView();
		
		var backBtn = view.query('#backButton')[0];
		var usernameInp = view.query('#usernameField')[0];
		var passwordInp = view.query('#passwordField')[0];
		var cPasswordInp = view.query('#cPasswordField')[0];
		
		var registerBtn = view.query('#registerButton')[0];
		
		//handler for back button
		if (!backBtn.hasListener('tap')) {
			backBtn.on('tap', function() {
				window.location = '#login';
			});
		}
		
		//register button handler
		if (!registerBtn.hasListener('tap')) {
			registerBtn.on('tap', function() {
				var username = usernameInp.getValue().trim(),
					password = passwordInp.getValue(),
					cPassword = cPasswordInp.getValue();
					
				if (!username) {
					alert('You must add a username');
					return;
				}
				
				if (!password) {
					alert('You must add a password');
					return;
				}
				
				if (cPassword != password) {
					alert('The two passwords must be the same');
					return;
				}
				
				socket.emit('register', username, password);
			});
		}
		
		this.showView(view);
	},
	
	//handlers
	//register response handler
	register_response: function(message, success) {
		//display message from server
		alert(message);
		
		//if register successful
		if (1 == success) {
			//redirect to login screen
			window.location = '#login';
		}
	},
	
	//login response handler
	login_response: function(message, success, username) {
		//if no success
		if (0 == success) {
			//display server message and stop function
			alert(message);
			return;
		}
		
		//create logged user object
		var loggedUser = {
			user_id: message,
			user_username: username
		};
		
		//save logged user to local storage
		localStorage.setItem('loggedUser', JSON.stringify(loggedUser));
		window.location = '#userslist';
	},
	
	//auto login handler
	auto_login_response: function (success) {
		//if no success from the server
		if (0 == success) {
			//display error message and redirect to login
			alert('An error occured while logging in. Please try again');
			window.location = '#login';
			return;
		}
		
		//if reached here, redirect to online users list
		window.location = '#userslist';
	}
 });
