var sql = require('./db');

//save message to database
var save_message = function(message_from, message_to, message_content) {
	var message_time = parseInt((new Date().getTime())/1000, 10);
	
	//query to insert message in database
	var query = 'INSERT INTO ' +
				sql.tablePrefix+'messages ' +
				'SET ' +
				'message_from = ?, ' +
				'message_to = ?, ' +
				'message_content = ?, ' +
				'message_time = ?';
		
	var newId = 0;
				
	sql.connection.query().execute(
		query, 
		[message_from, message_to, message_content, message_time],
		function (error, rows, cols, result) {
			if (error) {
				console.log(query);
				console.log(error);
				return;
			}
			
			newId = rows.id;
		},
		{'async': false}
	);
	
	//return the id of the new message
	return newId;
};

//mark given messages as read
var read_messages = function(messages) {
	var limit = messages.length;
	
	//update all messages from array as read
	var query = 'UPDATE ' +
				sql.tablePrefix+'messages ' +
				'SET ' +
				'message_read = 1 ' +
				'WHERE ' +
				'message_id IN ? ' +
				'LIMIT ' + limit;
				
	sql.connection.query().execute(
		query,
		[messages],
		function(error, rows, cols) {
			if (error) {
				console.log(query);
				console.log(error);
				return;
			}
		}
	);
};

//get first no_elements messages of conversation starting from offset
var get_conv_messages = function(user_a, user_b, offset, no_elements) {
	//query to select the required messages
	var query = 'SELECT * ' +
				'FROM ' +
				sql.tablePrefix+'messages ' +
				'WHERE ' +
				'(message_from = ? AND message_to = ?) ' +
				'OR ' +
				'(message_from = ? AND message_to = ?) ' +
				'ORDER BY message_time DESC ' +
				'LIMIT ' + offset + ', ' + no_elements;
				
	var result = null;
	
	//execute the query
	sql.connection.query().execute(
		query,
		[user_a, user_b, user_b, user_a],
		function (error, rows, cols) {
			if (error) {
				console.log(query);
				console.log(error);
				return;
			}
			//if no error, put the result in returned var
			result = rows;
		},
		{'async': false}
	);
	
	return result;
};

exports.save_message = save_message;
exports.read_messages = read_messages;
exports.get_conv_messages = get_conv_messages;
